/*
 * PropertyChangeEmitter.java  $Revision: 1.3 $  $Date: 2007/10/03 04:59:32 $
 *
 * Copyright (C) 2007 Meldcraft Corporation
 *
 * This file is part of Scaffold.
 * 
 * Scaffold is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Scaffold is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Scaffold.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.meldcraft.application.guis.mvc;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JFormattedTextField;
import javax.swing.text.JTextComponent;

import com.meldcraft.application.ApplicationContext;
import com.meldcraft.util.ReflectUtil;

/**
 * @version $Revision: 1.3 $ $Date: 2007/10/03 04:59:32 $
 * @author Rob Richards
 */
public class PropertyChangeEmitter implements PropertyChangeListener
{
    private ApplicationContext  mContext;
    private String              mElementId;

    public PropertyChangeEmitter(ApplicationContext context, String elementId, Component comp)
    {
        mContext = context;
        mElementId = elementId;

        try
        {
            ReflectUtil.invokeMethod(comp, "addPropertyChangeListener", this);
        }
        catch (NoSuchMethodException e)
        {
            throw new RuntimeException("Could not addPropertyChangeListener", e);
        }
        catch (IllegalAccessException e)
        {
            throw new RuntimeException("Could not addPropertyChangeListener", e);
        }
        catch (InvocationTargetException e)
        {
            throw new RuntimeException("Could not addPropertyChangeListener", e);
        }
    }

    public void propertyChange(PropertyChangeEvent e)
    {
        Object source = e.getSource();
        String pname = e.getPropertyName();
        if ((pname != null) && pname.equals("value"))
        {
            if (source instanceof JFormattedTextField)
            {
                Object oldValue = e.getOldValue();
                Object newValue = e.getNewValue();
                if ((oldValue != null) || (newValue != null))
                {
                    if (newValue instanceof Number)
                    {
                        newValue = String.valueOf(newValue);
                    }
                    mContext.contextChanged(mElementId + ".valueChanged", newValue);
                }
            }
            else if (source instanceof JTextComponent)
            {
                JTextComponent text = (JTextComponent)source;
                mContext.contextChanged(mElementId + ".valueChanged", text.getText());
            }
        }
    }
}
